require "defines"
require("VVAR")
--[[ 
note: we don't restrict concrete placement under specialized building. No need to forbid that as if the building is here in the first place, it's because there is a platform under. 
But if the player remove the building, he will need to remove concrete first. (need to find a way to check for platform under concrete)
]]--



-- Look if the building is entirely on a platform.
local function check_building(entity,box)
	for x=entity.position.x+box.left_top.x,entity.position.x+box.right_bottom.x,1 do
		for y=entity.position.y+box.left_top.y,entity.position.y+box.right_bottom.y,1 do
			if(entity.surface.get_tile(x,y).name ~= "building-platform") then
				return true
			end
		end
	end
	return false
end
-- When player build something, check if it is a restricted building and build on a platform.
-- If not, remove it.
script.on_event(defines.events.on_built_entity, function(event)
	local player = game.players[event.player_index]
	local entity = event.created_entity
	local is_ghost = (entity.name == "entity-ghost");
	local entity_prototype = (is_ghost and entity.ghost_prototype or entity.prototype );
	local entity_should_be_remove = false
	local already_found = false
	
	for i=1,#disallowed_types do
		if entity_prototype.type == disallowed_types[i] then
			entity_should_be_remove = check_building(entity,entity_prototype.collision_box)
			already_found = true
		end
	end
	
	if(not entity_should_be_remove and not already_found) then
		for name,pattern in pairs(special_disallowed_types) do
			if entity_prototype.type == name and string.match(entity_prototype.name,pattern) == entity_prototype.name then
				entity_should_be_remove = check_building(entity,entity_prototype.collision_box)
				already_found = true
			end
		end
	end
	
	if(entity_should_be_remove) then
		-- when removing, we need to loot it in case another mod put item inside
		-- need a solution if some item can't go inside inventory, maybe spawn a chest ? For the moment they just vanish
		if(not is_ghost) then
			local loot = entity_prototype.loot
			if loot then
				for i,loot_ele in pairs(loot) do
					player.insert({name=loot.item,count=loot.count_max})
				end
			end
			player.insert({name=entity.prototype.name})  -- The entity by itself is not included in loot list
		end
		entity.destroy()
		--Tell the player why the building has been removed.
		player.print("This building cannot be build outside platform !")
	end
	
end)